# encoding: utf-8
from builtins import zip
import logging
from datetime import datetime

from pyramid.view import view_config
from pyramid.httpexceptions import HTTPFound
import ringo.lib.security as security
from ringo.lib.helpers import get_action_routename
from ringo.views.helpers import get_item_from_request
from ringo_printtemplate.views import print_template
from ec.views.report_sachbericht import DummyData
from ec.model.participant import Participant


log = logging.getLogger(__name__)


@view_config(route_name="participants-matching",
             renderer="matching.mako",
             permission="update",
             request_method="GET")
def matching_clearing(request):
    participant = get_item_from_request(request)
    if participant.participations:
        course = participant.participations[0].ecoi.course
    else:
        course = "-/-"
    matches = [mpt.matching_participant
               for mpt in participant.matching_participants]
    eccourses = [match.participations[0].ecoi.course for match in matches
                 if match.participations]

    def get_url(request, item):
        is_admin = request.user.has_role("admin")
        read_perm = security.has_permission('read', item, request)
        url = None
        if is_admin or read_perm:
            url = request.route_path(get_action_routename(Participant,
                                                          'read'),
                                     id=item.id)
        return url
    participant_urls = [get_url(request, item) for item in matches]
    return {"participant": participant,
            "participant_url": request.route_path(
                get_action_routename(Participant, 'read'),
                id=participant.id),
            "matches": list(zip(matches, eccourses, participant_urls)),
            "participant_course": course}


def get_german_transfer_msg(user):
    german_now = datetime.now().strftime("%d.%m.%Y, %H:%M")
    return (u"Übertragung entschieden durch {} {}, {}".format(
        user.profile[0].first_name, user.profile[0].last_name, german_now))


def get_german_checked_msg(user):
    german_now = datetime.now().strftime("%d.%m.%Y, %H:%M")

    return (u"Geprüft durch {} {}, {}".format(
        user.profile[0].first_name, user.profile[0].last_name, german_now))


@view_config(route_name="participants-matching",
             permission="update",
             request_method="POST")
def matching_clearing_POST(request):
    url = request.route_url('home')
    participant = get_item_from_request(request)
    if request.params.get("transfer"):
        transfer_id = int(request.params.get("transfer"))
        for mtp in participant.matching_participants:
            if mtp.status == 0:
                if mtp.matching_participant_id == transfer_id:
                    mtp.status = 2
                    mtp.comment = get_german_transfer_msg(request.user)
                else:
                    mtp.status = 1
                    mtp.comment = get_german_checked_msg(request.user)
                request.db.add(mtp)
        return HTTPFound(request.route_url(
            "participants-matching-transfer-success", id=participant.id))

    for mtp in participant.matching_participants:
        if mtp.status == 0:
            mtp.status = 1
            mtp.comment = get_german_checked_msg(request.user)
            request.db.add(mtp)
    msg = (u"Für Teilnehmenden '{}' mögliche Dubletten als überprüft "
           u"markiert.").format(participant)
    request.session.flash(msg, 'success')
    return HTTPFound(url)


@view_config(route_name="participants-matching-transfer-success",
             permission="update",
             request_method="GET",
             renderer="matching_transfer.mako")
def matching_transfer_success(request):
    participant = get_item_from_request(request)
    return {"participant": participant}


@view_config(route_name="participants-matching-download",
             permission="update",
             request_method="GET")
def matching_transfer_download(request):
    participant = get_item_from_request(request)
    for mpt in participant.matching_participants:
        if mpt.status == 2:
            pn_id = "-/-"
            if participant.participations[0]:
                pn_id = participant.participations[0].id
            mpt_pid = mpt.matching_participant.personal_id
            response = print_template(request,
                                      {"personal_id": participant.personal_id,
                                       "matching_personal_id": mpt_pid,
                                       "participation_id": pn_id},
                                      DummyData("@ec:ec/static/matching.odt",
                                                name="Transfer_Teilnahme"))
            return response
    return HTTPFound(request.route_url('home'))
